"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getAccountGpaHelpersFragment = void 0;
const shared_1 = require("../../../shared");
const visitors_1 = require("../../../visitors");
const ImportMap_1 = require("../ImportMap");
const common_1 = require("./common");
const valueNode_1 = require("./valueNode");
function getAccountGpaHelpersFragment(accountNode, programNode, typeManifestVisitor, byteSizeVisitor) {
    var _a;
    const imports = new ImportMap_1.ImportMap();
    // Discriminator.
    const { discriminator } = accountNode;
    let resolvedDiscriminator = null;
    if ((discriminator === null || discriminator === void 0 ? void 0 : discriminator.kind) === 'field') {
        const discriminatorField = accountNode.data.struct.fields.find((f) => f.name === discriminator.name);
        const discriminatorValue = ((_a = discriminatorField === null || discriminatorField === void 0 ? void 0 : discriminatorField.defaultsTo) === null || _a === void 0 ? void 0 : _a.value)
            ? (0, valueNode_1.getValueNodeFragment)(discriminatorField.defaultsTo.value)
            : undefined;
        if (discriminatorValue) {
            imports.mergeWith(discriminatorValue.imports);
            resolvedDiscriminator = {
                kind: 'field',
                name: discriminator.name,
                value: discriminatorValue.render,
            };
        }
    }
    else if ((discriminator === null || discriminator === void 0 ? void 0 : discriminator.kind) === 'size') {
        resolvedDiscriminator =
            accountNode.size !== undefined
                ? { kind: 'size', value: `${accountNode.size}` }
                : null;
    }
    // GPA Fields.
    const gpaFields = (0, shared_1.getGpaFieldsFromAccount)(accountNode, byteSizeVisitor).map((gpaField) => {
        const gpaFieldManifest = (0, visitors_1.visit)(gpaField.type, typeManifestVisitor);
        imports.mergeWith(gpaFieldManifest.looseType, gpaFieldManifest.encoder);
        return { ...gpaField, manifest: gpaFieldManifest };
    });
    let resolvedGpaFields = null;
    if (gpaFields.length > 0) {
        resolvedGpaFields = {
            type: `{ ${gpaFields
                .map((f) => `'${f.name}': ${f.manifest.looseType.render}`)
                .join(', ')} }`,
            argument: `{ ${gpaFields
                .map((f) => {
                const offset = f.offset === null ? 'null' : `${f.offset}`;
                return `'${f.name}': [${offset}, ${f.manifest.encoder.render}]`;
            })
                .join(', ')} }`,
        };
    }
    return (0, common_1.fragmentFromTemplate)('accountGpaHelpers.njk', {
        pascalCaseName: (0, shared_1.pascalCase)(accountNode.name),
        program: programNode,
        discriminator: resolvedDiscriminator,
        gpaFields: resolvedGpaFields,
    })
        .mergeImportsWith(imports)
        .addImports('shared', ['Context', 'gpaBuilder']);
}
exports.getAccountGpaHelpersFragment = getAccountGpaHelpersFragment;
//# sourceMappingURL=accountGpaHelpers.js.map