"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getAccountPdaHelpersFragment = void 0;
const shared_1 = require("../../../shared");
const visitors_1 = require("../../../visitors");
const ImportMap_1 = require("../ImportMap");
const common_1 = require("./common");
const valueNode_1 = require("./valueNode");
function getAccountPdaHelpersFragment(accountNode, programNode, typeManifestVisitor) {
    if (accountNode.seeds.length === 0) {
        return (0, common_1.fragment)('');
    }
    // Seeds.
    const imports = new ImportMap_1.ImportMap();
    const seeds = accountNode.seeds.map((seed) => {
        if (seed.kind === 'constant') {
            const seedManifest = (0, visitors_1.visit)(seed.type, typeManifestVisitor);
            imports.mergeWith(seedManifest.encoder);
            const seedValue = seed.value;
            const valueManifest = (0, valueNode_1.getValueNodeFragment)(seedValue);
            seedValue.render = valueManifest.render;
            imports.mergeWith(valueManifest.imports);
            return { ...seed, typeManifest: seedManifest };
        }
        if (seed.kind === 'variable') {
            const seedManifest = (0, visitors_1.visit)(seed.type, typeManifestVisitor);
            imports.mergeWith(seedManifest.looseType, seedManifest.encoder);
            return { ...seed, typeManifest: seedManifest };
        }
        imports.add('solanaAddresses', 'getAddressEncoder');
        return seed;
    });
    const hasVariableSeeds = accountNode.seeds.filter((seed) => seed.kind === 'variable').length > 0;
    return (0, common_1.fragmentFromTemplate)('accountPdaHelpers.njk', {
        pascalCaseName: (0, shared_1.pascalCase)(accountNode.name),
        program: programNode,
        seeds,
        hasVariableSeeds,
    })
        .mergeImportsWith(imports)
        .addImports('solanaAddresses', ['ProgramDerivedAddress'])
        .addImports('shared', [
        'Context',
        'FetchEncodedAccountOptions',
        'getProgramAddress',
        'getProgramDerivedAddress',
    ]);
}
exports.getAccountPdaHelpersFragment = getAccountPdaHelpersFragment;
//# sourceMappingURL=accountPdaHelpers.js.map