"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Fragment = exports.mergeFragments = exports.fragmentWithContextMap = exports.fragmentFromTemplate = exports.fragment = void 0;
const utils_1 = require("../../utils");
const ImportMap_1 = require("../ImportMap");
const ContextMap_1 = require("../ContextMap");
function fragment(render, imports) {
    return new Fragment(render, imports);
}
exports.fragment = fragment;
function fragmentFromTemplate(fragmentFile, context, options) {
    return fragment((0, utils_1.resolveTemplate)(`${__dirname}/..`, `fragments/${fragmentFile}`, context, options));
}
exports.fragmentFromTemplate = fragmentFromTemplate;
function fragmentWithContextMap(render, imports) {
    const f = fragment(render, imports);
    f.interfaces = new ContextMap_1.ContextMap();
    return f;
}
exports.fragmentWithContextMap = fragmentWithContextMap;
function mergeFragments(fragments, mergeRenders) {
    return new Fragment(mergeRenders(fragments.map((f) => f.render)), new ImportMap_1.ImportMap().mergeWith(...fragments));
}
exports.mergeFragments = mergeFragments;
class Fragment {
    constructor(render, imports) {
        this.render = render;
        this.imports = imports !== null && imports !== void 0 ? imports : new ImportMap_1.ImportMap();
    }
    setRender(render) {
        this.render = render;
        return this;
    }
    mapRender(fn) {
        this.render = fn(this.render);
        return this;
    }
    addImports(module, imports) {
        this.imports.add(module, imports);
        return this;
    }
    removeImports(module, imports) {
        this.imports.remove(module, imports);
        return this;
    }
    mergeImportsWith(...others) {
        this.imports.mergeWith(...others);
        return this;
    }
    addImportAlias(module, name, alias) {
        this.imports.addAlias(module, name, alias);
        return this;
    }
    clone() {
        return new Fragment(this.render).mergeImportsWith(this.imports);
    }
    toString() {
        return this.render;
    }
}
exports.Fragment = Fragment;
//# sourceMappingURL=common.js.map