"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getInstructionAccountTypeParamFragment = void 0;
const shared_1 = require("../../../shared");
const ImportMap_1 = require("../ImportMap");
const common_1 = require("./common");
function getInstructionAccountTypeParamFragment(instructionNode, instructionAccountNode, programNode, allowAccountMeta) {
    const pascalCaseName = (0, shared_1.pascalCase)(instructionAccountNode.name);
    const typeParam = `TAccount${pascalCaseName}`;
    const accountMeta = allowAccountMeta ? ' | IAccountMeta<string>' : '';
    const imports = new ImportMap_1.ImportMap();
    if (allowAccountMeta) {
        imports.add('solanaInstructions', 'IAccountMeta');
    }
    if (instructionNode.optionalAccountStrategy === 'omitted' &&
        instructionAccountNode.isOptional) {
        return (0, common_1.fragment)(`${typeParam} extends string${accountMeta} | undefined = undefined`, imports);
    }
    const defaultAddress = getDefaultAddress(instructionAccountNode.defaultsTo, programNode.publicKey);
    return (0, common_1.fragment)(`${typeParam} extends string${accountMeta} = ${defaultAddress}`, imports);
}
exports.getInstructionAccountTypeParamFragment = getInstructionAccountTypeParamFragment;
function getDefaultAddress(defaultsTo, programId) {
    switch (defaultsTo === null || defaultsTo === void 0 ? void 0 : defaultsTo.kind) {
        case 'publicKey':
            return `"${defaultsTo.publicKey}"`;
        case 'program':
            return `"${defaultsTo.program.publicKey}"`;
        case 'programId':
            return `"${programId}"`;
        default:
            return `string`;
    }
}
//# sourceMappingURL=instructionAccountTypeParam.js.map