"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getInstructionBytesCreatedOnChainFragment = void 0;
const shared_1 = require("../../../shared");
const ContextMap_1 = require("../ContextMap");
const ImportMap_1 = require("../ImportMap");
const common_1 = require("./common");
function getInstructionBytesCreatedOnChainFragment(instructionNode) {
    const bytes = instructionNode.bytesCreatedOnChain;
    const imports = new ImportMap_1.ImportMap();
    const interfaces = new ContextMap_1.ContextMap();
    if (bytes && 'includeHeader' in bytes && bytes.includeHeader) {
        imports.add('shared', 'ACCOUNT_HEADER_SIZE');
    }
    if ((bytes === null || bytes === void 0 ? void 0 : bytes.kind) === 'account') {
        const importFrom = bytes.importFrom === 'generated' ? 'generatedAccounts' : bytes.importFrom;
        imports.add(importFrom, `get${(0, shared_1.pascalCase)(bytes.name)}Size`);
    }
    else if ((bytes === null || bytes === void 0 ? void 0 : bytes.kind) === 'resolver') {
        imports.add(bytes.importFrom, (0, shared_1.camelCase)(bytes.name));
        interfaces.add(['getProgramAddress', 'getProgramDerivedAddress']);
    }
    const bytesFragment = (0, common_1.fragmentFromTemplate)('instructionBytesCreatedOnChain.njk', { bytes }).mergeImportsWith(imports);
    bytesFragment.interfaces = interfaces;
    return bytesFragment;
}
exports.getInstructionBytesCreatedOnChainFragment = getInstructionBytesCreatedOnChainFragment;
//# sourceMappingURL=instructionBytesCreatedOnChain.js.map