"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getInstructionFunctionLowLevelFragment = void 0;
const shared_1 = require("../../../shared");
const ImportMap_1 = require("../ImportMap");
const common_1 = require("./common");
const instructionAccountTypeParam_1 = require("./instructionAccountTypeParam");
function getInstructionFunctionLowLevelFragment(instructionNode, programNode, dataArgsManifest) {
    const imports = new ImportMap_1.ImportMap();
    const hasAccounts = instructionNode.accounts.length > 0;
    const hasLegacyOptionalAccounts = instructionNode.optionalAccountStrategy === 'omitted' &&
        instructionNode.accounts.some((account) => account.isOptional);
    const hasData = !!instructionNode.dataArgs.link ||
        instructionNode.dataArgs.struct.fields.length > 0;
    const hasArgs = !!instructionNode.dataArgs.link ||
        instructionNode.dataArgs.struct.fields.filter((field) => { var _a; return ((_a = field.defaultsTo) === null || _a === void 0 ? void 0 : _a.strategy) !== 'omitted'; }).length > 0;
    const argsType = instructionNode.dataArgs.link
        ? dataArgsManifest.looseType.render
        : `${(0, shared_1.pascalCase)(instructionNode.dataArgs.name)}Args`;
    const encoderFunction = instructionNode.dataArgs.link
        ? dataArgsManifest.encoder.render
        : `get${(0, shared_1.pascalCase)(instructionNode.dataArgs.name)}Encoder()`;
    if (instructionNode.dataArgs.link) {
        imports.mergeWith(dataArgsManifest.looseType, dataArgsManifest.encoder);
    }
    const accountTypeParamsFragment = (0, common_1.mergeFragments)(instructionNode.accounts.map((account) => (0, instructionAccountTypeParam_1.getInstructionAccountTypeParamFragment)(instructionNode, account, programNode, true)), (renders) => renders.join(', '));
    const accounts = instructionNode.accounts.map((account) => {
        const typeParam = `TAccount${(0, shared_1.pascalCase)(account.name)}`;
        const defaultValue = getDefaultValue(account, programNode, instructionNode.optionalAccountStrategy === 'omitted');
        const isOptionalOrHasLowLevelDefaultValues = account.isOptional || defaultValue !== null;
        return {
            ...account,
            typeParam,
            defaultRole: getDefaultRole(account),
            defaultValue,
            isOptionalOrHasLowLevelDefaultValues,
        };
    });
    const fragment = (0, common_1.fragmentFromTemplate)('instructionFunctionLowLevel.njk', {
        instruction: instructionNode,
        program: programNode,
        hasAccounts,
        hasLegacyOptionalAccounts,
        accounts,
        hasData,
        hasArgs,
        argsType,
        encoderFunction,
        accountTypeParams: accountTypeParamsFragment.render,
    })
        .mergeImportsWith(imports, accountTypeParamsFragment)
        .addImports('solanaAddresses', ['Base58EncodedAddress'])
        .addImports('solanaInstructions', ['IAccountMeta']);
    if (hasAccounts) {
        fragment
            .addImports('shared', ['accountMetaWithDefault'])
            .addImports('solanaInstructions', ['AccountRole']);
    }
    return fragment;
}
exports.getInstructionFunctionLowLevelFragment = getInstructionFunctionLowLevelFragment;
function getDefaultRole(account) {
    if (account.isSigner === true && account.isWritable) {
        return 'AccountRole.WRITABLE_SIGNER';
    }
    if (account.isSigner === true) {
        return 'AccountRole.READONLY_SIGNER';
    }
    if (account.isWritable) {
        return 'AccountRole.WRITABLE';
    }
    return 'AccountRole.READONLY';
}
function getDefaultValue(account, program, usesLegacyOptionalAccounts) {
    var _a, _b, _c;
    if (account.isOptional && usesLegacyOptionalAccounts) {
        return null;
    }
    if (account.isOptional || ((_a = account.defaultsTo) === null || _a === void 0 ? void 0 : _a.kind) === 'programId') {
        return `{ address: "${program.publicKey}" as Base58EncodedAddress<"${program.publicKey}">, role: AccountRole.READONLY }`;
    }
    if (((_b = account.defaultsTo) === null || _b === void 0 ? void 0 : _b.kind) === 'program') {
        return `{ address: "${account.defaultsTo.program.publicKey}" as Base58EncodedAddress<"${account.defaultsTo.program.publicKey}">, role: AccountRole.READONLY }`;
    }
    if (((_c = account.defaultsTo) === null || _c === void 0 ? void 0 : _c.kind) === 'publicKey') {
        return `"${account.defaultsTo.publicKey}"`;
    }
    return null;
}
//# sourceMappingURL=instructionFunctionLowLevel.js.map