"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getInstructionInputDefaultFragment = void 0;
const shared_1 = require("../../../shared");
const common_1 = require("./common");
const valueNode_1 = require("./valueNode");
function getInstructionInputDefaultFragment(input, optionalAccountStrategy, accountObject = 'accounts', argObject = 'args') {
    if (!input.defaultsTo) {
        return (0, common_1.fragmentWithContextMap)('');
    }
    const { defaultsTo } = input;
    const defaultFragment = (defaultValue, isWritable) => {
        const inputName = (0, shared_1.camelCase)(input.name);
        if (input.kind === 'account' && defaultsTo.kind === 'resolver') {
            return (0, common_1.fragmentWithContextMap)(`${accountObject}.${inputName} = { ...${accountObject}.${inputName}, ...${defaultValue} };`);
        }
        if (input.kind === 'account' && isWritable === undefined) {
            return (0, common_1.fragmentWithContextMap)(`${accountObject}.${inputName}.value = ${defaultValue};`);
        }
        if (input.kind === 'account') {
            return (0, common_1.fragmentWithContextMap)(`${accountObject}.${inputName}.value = ${defaultValue};\n` +
                `${accountObject}.${inputName}.isWritable = ${isWritable ? 'true' : 'false'}`);
        }
        return (0, common_1.fragmentWithContextMap)(`${argObject}.${inputName} = ${defaultValue};`);
    };
    switch (defaultsTo.kind) {
        case 'account':
            const name = (0, shared_1.camelCase)(defaultsTo.name);
            if (input.kind === 'account' &&
                input.resolvedIsSigner &&
                !input.isSigner) {
                return defaultFragment(`expectSigner(${accountObject}.${name}.value).address`).addImports('shared', 'expectSigner');
            }
            if (input.kind === 'account') {
                return defaultFragment(`expectSome(${accountObject}.${name}.value)`).addImports('shared', 'expectSome');
            }
            return defaultFragment(`expectAddress(${accountObject}.${name}.value)`).addImports('shared', 'expectAddress');
        case 'pda':
            const pdaFunction = `find${(0, shared_1.pascalCase)(defaultsTo.pdaAccount)}Pda`;
            const pdaImportFrom = defaultsTo.importFrom === 'generated'
                ? 'generatedAccounts'
                : defaultsTo.importFrom;
            const pdaArgs = ['context'];
            const pdaSeeds = Object.keys(defaultsTo.seeds).map((seed) => {
                const seedValue = defaultsTo.seeds[seed];
                if (seedValue.kind === 'account') {
                    return (0, common_1.fragment)(`${seed}: expectAddress(${accountObject}.${(0, shared_1.camelCase)(seedValue.name)}.value)`).addImports('shared', 'expectAddress');
                }
                if (seedValue.kind === 'arg') {
                    return (0, common_1.fragment)(`${seed}: expectSome(${argObject}.${(0, shared_1.camelCase)(seedValue.name)})`).addImports('shared', 'expectSome');
                }
                return (0, valueNode_1.getValueNodeFragment)(seedValue.value).mapRender((r) => `${seed}: ${r}`);
            });
            const pdaSeedsFragment = (0, common_1.mergeFragments)(pdaSeeds, (renders) => renders.join(', ')).mapRender((r) => `{ ${r} }`);
            if (pdaSeeds.length > 0) {
                pdaArgs.push(pdaSeedsFragment.render);
            }
            const pdaFragment = defaultFragment(`await ${pdaFunction}(${pdaArgs.join(', ')})`)
                .mergeImportsWith(pdaSeedsFragment)
                .addImports(pdaImportFrom, pdaFunction);
            pdaFragment.interfaces.add('getProgramDerivedAddress');
            return pdaFragment;
        case 'publicKey':
            return defaultFragment(`'${defaultsTo.publicKey}' as Base58EncodedAddress<'${defaultsTo.publicKey}'>`).addImports('solanaAddresses', 'Base58EncodedAddress');
        case 'program':
            const programFragment = defaultFragment(`await getProgramAddress(context, '${defaultsTo.program.name}', '${defaultsTo.program.publicKey}')`, false).addImports('shared', ['getProgramAddress']);
            programFragment.interfaces.add('getProgramAddress');
            return programFragment;
        case 'programId':
            if (optionalAccountStrategy === 'programId' &&
                input.kind === 'account' &&
                input.isOptional) {
                return (0, common_1.fragmentWithContextMap)('');
            }
            return defaultFragment('programAddress', false);
        case 'identity':
        case 'payer':
            return (0, common_1.fragmentWithContextMap)('');
        case 'accountBump':
            return defaultFragment(`expectProgramDerivedAddress(${accountObject}.${(0, shared_1.camelCase)(defaultsTo.name)}.value)[1]`).addImports('shared', 'expectProgramDerivedAddress');
        case 'arg':
            return defaultFragment(`expectSome(${argObject}.${(0, shared_1.camelCase)(defaultsTo.name)})`).addImports('shared', 'expectSome');
        case 'value':
            const valueManifest = (0, valueNode_1.getValueNodeFragment)(defaultsTo.value);
            return defaultFragment(valueManifest.render).mergeImportsWith(valueManifest);
        case 'resolver':
            const resolverName = (0, shared_1.camelCase)(defaultsTo.name);
            const isWritable = input.kind === 'account' && input.isWritable ? 'true' : 'false';
            const resolverFragment = defaultFragment(`await ${resolverName}(context, ${accountObject}, ${argObject}, programAddress, ${isWritable})`).addImports(defaultsTo.importFrom, resolverName);
            resolverFragment.interfaces.add([
                'getProgramAddress',
                'getProgramDerivedAddress',
            ]);
            return resolverFragment;
        case 'conditional':
        case 'conditionalResolver':
            const ifTrueRenderer = renderNestedInstructionDefault(input, optionalAccountStrategy, defaultsTo.ifTrue, accountObject, argObject);
            const ifFalseRenderer = renderNestedInstructionDefault(input, optionalAccountStrategy, defaultsTo.ifFalse, accountObject, argObject);
            if (!ifTrueRenderer && !ifFalseRenderer) {
                return (0, common_1.fragmentWithContextMap)('');
            }
            const conditionalFragment = (0, common_1.fragmentWithContextMap)('');
            if (ifTrueRenderer) {
                conditionalFragment.mergeImportsWith(ifTrueRenderer.imports);
                conditionalFragment.interfaces.mergeWith(ifTrueRenderer.interfaces);
            }
            if (ifFalseRenderer) {
                conditionalFragment.mergeImportsWith(ifFalseRenderer.imports);
                conditionalFragment.interfaces.mergeWith(ifFalseRenderer.interfaces);
            }
            const negatedCondition = !ifTrueRenderer;
            let condition = 'true';
            if (defaultsTo.kind === 'conditional') {
                const comparedInputName = defaultsTo.input.kind === 'account'
                    ? `${accountObject}.${(0, shared_1.camelCase)(defaultsTo.input.name)}.value`
                    : `${argObject}.${(0, shared_1.camelCase)(defaultsTo.input.name)}`;
                if (defaultsTo.value) {
                    const comparedValue = (0, valueNode_1.getValueNodeFragment)(defaultsTo.value);
                    conditionalFragment.mergeImportsWith(comparedValue.imports);
                    const operator = negatedCondition ? '!==' : '===';
                    condition = `${comparedInputName} ${operator} ${comparedValue.render}`;
                }
                else {
                    condition = negatedCondition
                        ? `!${comparedInputName}`
                        : comparedInputName;
                }
            }
            else {
                const conditionalResolverName = (0, shared_1.camelCase)(defaultsTo.resolver.name);
                const conditionalIsWritable = input.kind === 'account' && input.isWritable ? 'true' : 'false';
                conditionalFragment.addImports(defaultsTo.resolver.importFrom, conditionalResolverName);
                conditionalFragment.interfaces.add([
                    'getProgramAddress',
                    'getProgramDerivedAddress',
                ]);
                condition = `await ${conditionalResolverName}(context, ${accountObject}, ${argObject}, programAddress, ${conditionalIsWritable})`;
                condition = negatedCondition ? `!${condition}` : condition;
            }
            if (ifTrueRenderer && ifFalseRenderer) {
                return conditionalFragment.setRender(`if (${condition}) {\n${ifTrueRenderer.render}\n} else {\n${ifFalseRenderer.render}\n}`);
            }
            return conditionalFragment.setRender(`if (${condition}) {\n${ifTrueRenderer ? ifTrueRenderer.render : ifFalseRenderer === null || ifFalseRenderer === void 0 ? void 0 : ifFalseRenderer.render}\n}`);
        default:
            const neverDefault = defaultsTo;
            throw new Error(`Unexpected value type ${neverDefault.kind}`);
    }
}
exports.getInstructionInputDefaultFragment = getInstructionInputDefaultFragment;
function renderNestedInstructionDefault(input, optionalAccountStrategy, defaultsTo, accountObject, argObject) {
    if (!defaultsTo)
        return undefined;
    if (input.kind === 'account') {
        return getInstructionInputDefaultFragment({ ...input, defaultsTo: defaultsTo }, optionalAccountStrategy, accountObject, argObject);
    }
    return getInstructionInputDefaultFragment({ ...input, defaultsTo: defaultsTo }, optionalAccountStrategy, accountObject, argObject);
}
//# sourceMappingURL=instructionInputDefault.js.map