"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getInstructionInputResolvedFragment = void 0;
const shared_1 = require("../../../shared");
const ContextMap_1 = require("../ContextMap");
const common_1 = require("./common");
const instructionInputDefault_1 = require("./instructionInputDefault");
function getInstructionInputResolvedFragment(instructionNode, resolvedInputs) {
    const interfaces = new ContextMap_1.ContextMap();
    const resolvedInputFragments = resolvedInputs.flatMap((input) => {
        const inputFragment = (0, instructionInputDefault_1.getInstructionInputDefaultFragment)(input, instructionNode.optionalAccountStrategy);
        if (!inputFragment.render)
            return [];
        interfaces.mergeWith(inputFragment.interfaces);
        const camelName = (0, shared_1.camelCase)(input.name);
        return [
            inputFragment.mapRender((r) => input.kind === 'arg'
                ? `if (!args.${camelName}) {\n${r}\n}`
                : `if (!accounts.${camelName}.value) {\n${r}\n}`),
        ];
    });
    if (resolvedInputFragments.length === 0) {
        return (0, common_1.fragmentWithContextMap)('');
    }
    const resolvedInputsFragment = (0, common_1.mergeFragments)([(0, common_1.fragment)('// Resolve default values.'), ...resolvedInputFragments], (renders) => renders.join('\n'));
    resolvedInputsFragment.interfaces = interfaces;
    return resolvedInputsFragment;
}
exports.getInstructionInputResolvedFragment = getInstructionInputResolvedFragment;
//# sourceMappingURL=instructionInputResolved.js.map