"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getInstructionTypeFragment = void 0;
const shared_1 = require("../../../shared");
const common_1 = require("./common");
const instructionAccountMeta_1 = require("./instructionAccountMeta");
const instructionAccountTypeParam_1 = require("./instructionAccountTypeParam");
function getInstructionTypeFragment(instructionNode, programNode) {
    const hasAccounts = instructionNode.accounts.length > 0;
    const hasData = !!instructionNode.dataArgs.link ||
        instructionNode.dataArgs.struct.fields.length > 0;
    const dataType = instructionNode.dataArgs.link
        ? (0, shared_1.pascalCase)(instructionNode.dataArgs.link.name)
        : (0, shared_1.pascalCase)(instructionNode.dataArgs.name);
    const accountTypeParamsFragment = (0, common_1.mergeFragments)(instructionNode.accounts.map((account) => (0, instructionAccountTypeParam_1.getInstructionAccountTypeParamFragment)(instructionNode, account, programNode, true)), (renders) => renders.join(', '));
    const usesLegacyOptionalAccounts = instructionNode.optionalAccountStrategy === 'omitted';
    const accountMetasFragment = (0, common_1.mergeFragments)(instructionNode.accounts.map((account) => (0, instructionAccountMeta_1.getInstructionAccountMetaFragment)(account).mapRender((r) => {
        const typeParam = `TAccount${(0, shared_1.pascalCase)(account.name)}`;
        const isLegacyOptional = account.isOptional && usesLegacyOptionalAccounts;
        const type = `${typeParam} extends string ? ${r} : ${typeParam}`;
        if (!isLegacyOptional)
            return type;
        return `...(${typeParam} extends undefined ? [] : [${type}])`;
    })), (renders) => renders.join(', '));
    const fragment = (0, common_1.fragmentFromTemplate)('instructionType.njk', {
        instruction: instructionNode,
        program: programNode,
        hasData,
        hasAccounts,
        dataType,
        accountTypeParams: accountTypeParamsFragment.render,
        accountMetas: accountMetasFragment.render,
    })
        .mergeImportsWith(accountTypeParamsFragment, accountMetasFragment)
        .addImports('solanaInstructions', [
        'IAccountMeta',
        'IInstruction',
        'IInstructionWithAccounts',
        ...(hasData ? ['IInstructionWithData'] : []),
    ]);
    // TODO: if link, add import for data type. Unless we don't need to inject the data type in IInstructionWithData.
    return fragment;
}
exports.getInstructionTypeFragment = getInstructionTypeFragment;
//# sourceMappingURL=instructionType.js.map