import type { ConfigureOptions } from 'nunjucks';
import { Options as PrettierOptions } from 'prettier';
import * as nodes from '../../nodes';
import { ImportFrom } from '../../shared';
import { BaseThrowVisitor, ResolvedInstructionAccount, ResolvedInstructionInput, Visitor } from '../../visitors';
import { RenderMap } from '../RenderMap';
import { ImportMap } from './ImportMap';
import { TypeManifest } from './TypeManifest';
export type GetRenderMapOptions = {
    renderParentInstructions?: boolean;
    formatCode?: boolean;
    prettierOptions?: PrettierOptions;
    dependencyMap?: Record<ImportFrom, string>;
    typeManifestVisitor?: Visitor<TypeManifest>;
    byteSizeVisitor?: Visitor<number | null> & {
        registerDefinedTypes?: (definedTypes: nodes.DefinedTypeNode[]) => void;
    };
    resolvedInstructionInputVisitor?: Visitor<ResolvedInstructionInput[]>;
};
export declare class GetRenderMapVisitor extends BaseThrowVisitor<RenderMap> {
    readonly options: Required<GetRenderMapOptions>;
    private program;
    constructor(options?: GetRenderMapOptions);
    visitRoot(root: nodes.RootNode): RenderMap;
    visitProgram(program: nodes.ProgramNode): RenderMap;
    visitAccount(account: nodes.AccountNode): RenderMap;
    visitInstruction(instruction: nodes.InstructionNode): RenderMap;
    visitDefinedType(definedType: nodes.DefinedTypeNode): RenderMap;
    get typeManifestVisitor(): Visitor<TypeManifest>;
    get byteSizeVisitor(): Visitor<number | null> & {
        registerDefinedTypes?: ((definedTypes: nodes.DefinedTypeNode[]) => void) | undefined;
    };
    get resolvedInstructionInputVisitor(): Visitor<ResolvedInstructionInput[]>;
    protected getInstructionAccountType(account: ResolvedInstructionAccount): string;
    protected getInstructionAccountImports(accounts: ResolvedInstructionAccount[]): ImportMap;
    protected getMergeConflictsForInstructionAccountsAndArgs(instruction: nodes.InstructionNode): string[];
    protected getRenamedArgsMap(instruction: nodes.InstructionNode): Map<string, string>;
    protected render(template: string, context?: object, options?: ConfigureOptions): string;
}
